package com.caliverse.admin.dynamodb.repository;

import com.caliverse.admin.domain.request.LandRequest;
import com.caliverse.admin.dynamodb.domain.atrrib.LandAuctionRegistryAttrib;
import com.caliverse.admin.dynamodb.domain.doc.LandAuctionRegistryDoc;
import com.caliverse.admin.dynamodb.entity.DynamodbOperationResult;

public interface LandAuctionRegistryRepository extends DynamoDBRepository<LandAuctionRegistryDoc> {
    LandAuctionRegistryAttrib findRegistryAttrib(Integer landId, Integer auctionSeq);
    void cancelAuction(Integer landId, Integer auctionSeq);
    void insertAuction(LandRequest landRequest);
    void updateAuction(LandRequest landRequest);
    int findAuctionNumber(Integer landId);
    DynamodbOperationResult initLandAuctionRegistry(Integer landId, Integer auctionSeq);
}
